import isEqual from 'lodash/isEqual';
import isEmpty from 'lodash/isEmpty';
import { PureComponent, Children, cloneElement } from 'react';
import PropTypes from 'prop-types';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var handleCallback = function handleCallback(resolve, reject, callback, r) {
  try {
    resolve(callback(r));
  } catch (e) {
    reject(e);
  }
};

var CancelablePromise = function () {
  createClass(CancelablePromise, null, [{
    key: "all",
    value: function all(iterable) {
      return new CancelablePromise(function (y, n) {
        Promise.all(iterable).then(y, n);
      });
    }
  }, {
    key: "race",
    value: function race(iterable) {
      return new CancelablePromise(function (y, n) {
        Promise.race(iterable).then(y, n);
      });
    }
  }, {
    key: "reject",
    value: function reject(value) {
      return new CancelablePromise(function (y, n) {
        Promise.reject(value).then(y, n);
      });
    }
  }, {
    key: "resolve",
    value: function resolve(value) {
      return new CancelablePromise(function (y, n) {
        Promise.resolve(value).then(y, n);
      });
    }
  }]);

  function CancelablePromise(executor) {
    classCallCheck(this, CancelablePromise);

    this._promise = new Promise(executor);
    this._canceled = false;
  }

  createClass(CancelablePromise, [{
    key: "then",
    value: function then(success, error) {
      var _this = this;

      var p = new CancelablePromise(function (resolve, reject) {
        _this._promise.then(function (r) {
          if (_this._canceled) {
            p.cancel();
          }
          if (success && !_this._canceled) {
            handleCallback(resolve, reject, success, r);
          } else {
            resolve(r);
          }
        }, function (r) {
          if (_this._canceled) {
            p.cancel();
          }
          if (error && !_this._canceled) {
            handleCallback(resolve, reject, error, r);
          } else {
            reject(r);
          }
        });
      });
      return p;
    }
  }, {
    key: "catch",
    value: function _catch(error) {
      return this.then(undefined, error);
    }
  }, {
    key: "cancel",
    value: function cancel(errorCallback) {
      this._canceled = true;
      if (errorCallback) {
        this._promise.catch(errorCallback);
      }
      return this;
    }
  }]);
  return CancelablePromise;
}();

var LayerModel = function () {
  function LayerModel() {
    var layerSpec = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    classCallCheck(this, LayerModel);
    this.opacity = 1;
    this.visibility = true;

    Object.assign(this, layerSpec, { changedAttributes: {} });
  }

  createClass(LayerModel, [{
    key: 'get',
    value: function get$$1(key) {
      return this[key];
    }
  }, {
    key: 'set',
    value: function set$$1(key, value) {
      this[key] = value;
      return this;
    }
  }, {
    key: 'update',
    value: function update(layerSpec) {
      var _this = this;

      var prevData = _extends({}, this);
      var nextData = _extends({}, layerSpec);

      // reseting changedAttributes for every update
      this.set('changedAttributes', {});

      Object.keys(nextData).forEach(function (k) {
        if (!isEqual(prevData[k], nextData[k])) {
          _this.changedAttributes[k] = nextData[k];
          _this.set(k, nextData[k]);
        }
      });
    }
  }]);
  return LayerModel;
}();

function checkPluginProperties(plugin) {
  if (plugin) {
    var requiredProperties = ['add', 'remove', 'setVisibility', 'setOpacity', 'setZIndex', 'setLayerConfig', 'setParams', 'setSQLParams', 'setDecodeParams', 'getLayerByType'];

    requiredProperties.forEach(function (property) {
      if (!plugin[property]) {
        console.error('The ' + property + ' function is required for layer manager plugins');
      }
    });

    var deprecatedProperties = ['getLayerByProvider'];
    deprecatedProperties.forEach(function (property) {
      if (plugin[property]) {
        console.warn('The ' + property + ' function is deprecated for layer manager plugins');
      }
    });
  }
}

var LayerManager = function () {
  function LayerManager(map, Plugin) {
    classCallCheck(this, LayerManager);

    this.map = map;
    this.layers = [];
    this.promises = {};
    this.options = {
      getLayers: this.getLayers.bind(this)
    };

    this.plugin = new Plugin(this.map, this.options);
    checkPluginProperties(this.plugin);
  }

  /**
   * Add layers
   * @param {Array} layers
   * @param {Object} layerOptions
   */


  createClass(LayerManager, [{
    key: 'add',
    value: function add(layer) {
      var layerOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
        opacity: 1,
        visibility: true,
        zIndex: 0,
        interactivity: null
      };

      if (typeof layer === 'undefined') {
        console.error('layer is required');
        return this;
      }

      var layerModel = new LayerModel(_extends({}, layer, layerOptions));
      var id = layerModel.id;


      if (this.layers.find(function (l) {
        return l.id === id;
      })) {
        return this.layers;
      }
      this.layers.push(layerModel);

      this.requestLayer(layerModel);

      return this.layers;
    }

    /**
     * Updating a specific layer
     * @param  {Object} layerModel
     */

  }, {
    key: 'update',
    value: function update(id, changedProps) {
      var layerModel = this.getLayerModel(id);
      if (!layerModel || !layerModel.mapLayer) return;

      layerModel.update(changedProps);

      var opacity = changedProps.opacity,
          visibility = changedProps.visibility,
          zIndex = changedProps.zIndex,
          decodeParams = changedProps.decodeParams;


      if (typeof opacity !== 'undefined') {
        this.plugin.setOpacity(layerModel, opacity);
      }
      if (typeof visibility !== 'undefined') {
        this.plugin.setVisibility(layerModel, visibility);
      }
      if (typeof zIndex !== 'undefined') {
        this.plugin.setZIndex(layerModel, zIndex);
      }

      if (!isEmpty(decodeParams)) {
        this.plugin.setDecodeParams(layerModel);
      }
    }

    /**
     * Remove a layer giving a Layer ID
     * @param {String} id
     */

  }, {
    key: 'remove',
    value: function remove(id) {
      var layers = this.layers.slice(0);

      this.requestCancel(id);

      var layerModel = this.getLayerModel(id);

      if (layerModel) {
        this.plugin.remove(layerModel);
      }

      this.layers = layers.filter(function (l) {
        return l.id !== id;
      });
    }
  }, {
    key: 'getLayers',
    value: function getLayers() {
      return this.layers;
    }
  }, {
    key: 'getLayerModel',
    value: function getLayerModel(id) {
      return this.layers.find(function (l) {
        return l.id === id;
      });
    }

    /**
     * A namespace to set opacity on selected layer
     * @param {String} id
     * @param {Number} opacity
     */

  }, {
    key: 'setOpacity',
    value: function setOpacity(id, opacity) {
      var layerModel = this.getLayerModel(id);
      this.plugin.setOpacity(layerModel, opacity);
    }

    /**
     * A namespace to hide or show a selected layer
     * @param {String} id
     * @param {Boolean} visibility
     */

  }, {
    key: 'setVisibility',
    value: function setVisibility(id, visibility) {
      var layerModel = this.getLayerModel(id);
      this.plugin.setVisibility(layerModel, visibility);
    }

    /**
     * A namespace to set z-index on selected layer
     * @param {String} id
     * @param {Number} zIndex
     */

  }, {
    key: 'setZIndex',
    value: function setZIndex(id, zIndex) {
      var layerModel = this.getLayerModel(id);
      this.plugin.setZIndex(layerModel, zIndex);
    }
  }, {
    key: 'requestLayer',
    value: function requestLayer(layerModel) {
      var _this = this;

      var layerType = layerModel.layerType,
          provider = layerModel.provider;

      var method = this.plugin.getLayerByType(layerType) || this.plugin.getLayerByProvider(provider, layerModel);

      if (!method) {
        this.promises[layerModel.id] = CancelablePromise.reject(new Error(provider + ' provider is not yet supported.'));
        return false;
      }

      // Cancel previous/existing request
      this.requestCancel(layerModel.id);

      // every request method returns a promise that we store in the array
      // to control when all layers are fetched.
      this.promises[layerModel.id] = method.call(this, layerModel).then(function (layer) {
        var canceled = _this.promises[layerModel.id]._canceled;

        if (!canceled) {
          layerModel.set('mapLayer', layer);
          _this.plugin.add(layerModel, _this.layers);
          _this.plugin.setZIndex(layerModel, layerModel.zIndex);
          _this.plugin.setOpacity(layerModel, layerModel.opacity);
          _this.plugin.setVisibility(layerModel, layerModel.visibility);
        }
      });

      return this;
    }
  }, {
    key: 'requestCancel',
    value: function requestCancel(layerModelId) {
      // Cancel previous/existing request
      if (this.promises[layerModelId] && this.promises[layerModelId].cancel) {
        this.promises[layerModelId].cancel();
      }
    }
  }, {
    key: 'unmount',
    value: function unmount() {
      this.map = null;
      this.plugin.unmount();
    }
  }]);
  return LayerManager;
}();

var LayerManager$1 = function (_PureComponent) {
  inherits(LayerManager$$1, _PureComponent);

  function LayerManager$$1(props) {
    classCallCheck(this, LayerManager$$1);

    var _this = possibleConstructorReturn(this, (LayerManager$$1.__proto__ || Object.getPrototypeOf(LayerManager$$1)).call(this, props));

    var map = props.map,
        plugin = props.plugin;

    _this.layerManager = new LayerManager(map, plugin);
    return _this;
  }

  createClass(LayerManager$$1, [{
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.layerManager.unmount();
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var children = this.props.children;


      if (children && Children.count(children)) {
        return Children.map(children, function (child, i) {
          return child && cloneElement(child, {
            layerManager: _this2.layerManager,
            zIndex: child.props.zIndex || 1000 - i
          });
        });
      }

      return null;
    }
  }]);
  return LayerManager$$1;
}(PureComponent);

LayerManager$1.propTypes = {
  map: PropTypes.shape({}).isRequired,
  plugin: PropTypes.func.isRequired,
  children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.node), PropTypes.node])
};
LayerManager$1.defaultProps = {
  children: []
};

var Layer = function (_PureComponent) {
  inherits(Layer, _PureComponent);

  function Layer() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, Layer);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Layer.__proto__ || Object.getPrototypeOf(Layer)).call.apply(_ref, [this].concat(args))), _this), _this.add = function () {
      var _this$props = _this.props,
          layerManager = _this$props.layerManager,
          props = objectWithoutProperties(_this$props, ['layerManager']);

      layerManager.add(props, {});
    }, _this.update = function (changedProps) {
      var _this$props2 = _this.props,
          layerManager = _this$props2.layerManager,
          id = _this$props2.id;


      layerManager.update(id, changedProps);
    }, _this.remove = function () {
      var _this$props3 = _this.props,
          layerManager = _this$props3.layerManager,
          id = _this$props3.id;

      layerManager.remove(id);
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Layer, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.add();
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps) {
      var prevLayerConfig = prevProps.layerConfig,
          prevParams = prevProps.params,
          prevSqlParams = prevProps.sqlParams,
          prevDecodeParams = prevProps.decodeParams,
          prevOpacity = prevProps.opacity,
          prevVisibility = prevProps.visibility,
          prevZIndex = prevProps.zIndex;
      var _props = this.props,
          layerConfig = _props.layerConfig,
          params = _props.params,
          sqlParams = _props.sqlParams,
          decodeParams = _props.decodeParams,
          opacity = _props.opacity,
          visibility = _props.visibility,
          zIndex = _props.zIndex;


      if (layerConfig && !isEqual(layerConfig, prevLayerConfig) || params && !isEqual(params, prevParams) || sqlParams && !isEqual(sqlParams, prevSqlParams)) {
        this.remove();
        this.add();
      }

      var changedProps = _extends({}, opacity !== prevOpacity && {
        opacity: opacity
      }, visibility !== prevVisibility && {
        visibility: visibility
      }, zIndex !== prevZIndex && {
        zIndex: zIndex
      }, !isEqual(decodeParams, prevDecodeParams) && {
        decodeParams: decodeParams
      });

      if (!isEmpty(changedProps)) {
        this.update(changedProps);
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.remove();
    }
  }, {
    key: 'render',
    value: function render() {
      return null;
    }
  }]);
  return Layer;
}(PureComponent);

Layer.propTypes = {
  id: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,

  layerConfig: PropTypes.shape({}).isRequired,
  params: PropTypes.shape({}),
  sqlParams: PropTypes.shape({}),
  decodeParams: PropTypes.shape({}),

  opacity: PropTypes.number,
  visibility: PropTypes.bool,
  zIndex: PropTypes.number,

  layerManager: PropTypes.shape({
    add: PropTypes.func.isRequired,
    update: PropTypes.func.isRequired,
    remove: PropTypes.func.isRequired
  })
};
Layer.defaultProps = {
  params: undefined,
  sqlParams: undefined,
  decodeParams: undefined,
  opacity: 1,
  visibility: true,
  zIndex: undefined,

  layerManager: null
};

export { LayerManager$1 as LayerManager, Layer };
